# sb::App::Admin
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Admin;
use strict;
use sb::Data ();
use sb::Language ();
use sb::Text ();
use sb::File::Uploader ();
use sb::File::Object ();
use sb::Session ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.25';
# 0.26 [2009/03/19] modified set_message to handle linefeeds
# 0.25 [2009/02/07] modiifed _check_mode to set cgi object for sb::Session
# 0.24 [2009/02/03] modified extra_styleset to get extra styles for admin.cgi
# 0.23 [2009/01/24] modified upload_file to create thumnail without id
# 0.22 [2008/11/23] modified upload_file to deal with overwriting the existed files
# 0.21 [2008/11/16] renamed extra_styles to make it public
# 0.20 [2008/11/09] renamed _get_next_imageid from _get_latest_imageid
# 0.19 [2008/10/22] fixed a bug which registered user cannot see the closed articles
# 0.18 [2007/11/30] modified run to enable admin plugins
# 0.17 [2007/11/05] changed _set_title to use title_name
# 0.16 [2007/09/17] changed upload_file and added _get_latest_imageid
# 0.15 [2007/09/15] changed _check_mode to consider unknown mode
# 0.14 [2007/09/14] changed run to make sure "bye" is executed
# 0.13 [2007/09/13] changed default_menulist
# 0.12 [2007/09/12] added get_editor, editor_name, and js_paths
# 0.11 [2007/09/10] changed $cms to $temp to make them consitency
# 0.10 [2007/09/09] modified _set_title and _set_menu
# 0.09 [2007/09/08] fixed bugs in _check_mode which mode and regi weren't set
# 0.08 [2007/08/12] modified error strings
# 0.07 [2007/07/20] refactored for Serene Bach 3.*
# 0.06 [2006/02/03] changed %mAdminMode to enable rebuild menu for normal user
# 0.05 [2005/08/12] chnaged _check_mode to create session instance with name
# 0.04 [2005/08/11] changed _check_mode to check permission at proper timing
# 0.03 [2005/07/12] changed %mAdminMode to display help
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/06/07] changed set_main/_set_title/_set_menu to implement bookmarklet
# 0.00 [2004/02/01] generated
# --------------------------------------
# // defines
# --- for template
sub TEMPLATE        (){ 'main.html' };
sub JSFILE_NAME     (){ 'script/adminutils.js' }
sub CSS_NAME        (){ 'style.css' };
sub EXTRA_STYLE     (){ '<link rel="%s" href="%s" type="text/css" title="%s" media="screen,tv" />' };
sub CATEGORY_OPTION (){ '%Main% &gt; %Sub%' };
sub DEFAULT_EDITOR  (){ 'sb::Admin::Editor' };
# --- for cookie
sub COOKIE_USER     (){ 'user' };
sub COOKIE_FOR_LOG  (){ 365 };
# --- for menu list
sub MENU_MODE   (){ 0 };
sub MENU_TYPE   (){ 1 };
sub MENU_MODULE (){ 2 };
sub MENU_LEVEL  (){ 3 };
sub MENU_HELP   (){ 4 };
# --- for others
sub MAX_LEVEL   (){ 2 }; # permission check
sub PREFIX_NAME (){ 'file' }; # file prefix
# --------------------------------------
# // static variables
sub default_menulist
{
	return(
		# [mode name => mode type,module name,mode level,help pointer],
		['home'      => 'head',  'sb::Admin::Home',     0,'_'        ],
		['status'    => '',      'sb::Admin::Home',     0,'_'        ],
		['view'      => 'head',  'sb::Admin::Preview',  0,'_'        ],
		['logout'    => 'head',  'sb::Admin::Login',    0,'_'        ],
		['login'     => '',      'sb::Admin::Login',    0,'_'        ],
		['help'      => '',      'sb::Admin::Help',     0,'_'        ],
		['profile'   => '',      'sb::Admin::Profile',  0,'_menu7'   ],
		['edituser'  => '',      'sb::Admin::Profile',  2,'_menu7'   ],
		['new'       => 'edit',  'sb::Admin::Entry',    0,'_entry'   ],
		['edit'      => '',      'sb::Admin::Entry',    0,'_entry'   ],
		['list'      => 'edit',  'sb::Admin::EntryList',0,'_entry'   ],
		['category'  => 'edit',  'sb::Admin::Category', 1,'_category'],
		['upload'    => 'edit',  'sb::Admin::Upload',   0,'_menu4'   ],
		['amazon'    => 'edit',  'sb::Admin::Amazon',   0,'_aws'     ],
		['link'      => 'edit',  'sb::Admin::Link',     1,'_menu6'   ],
		['edittemp'  => 'edit',  'sb::Admin::Template', 1,'_menu13'  ],
		['comment'   => 'manage','sb::Admin::Message',  0,'_comment' ],
		['trackback' => 'manage','sb::Admin::Trackback',0,'_comment' ],
		['template'  => 'manage','sb::Admin::Template', 1,'_menu13'  ],
		['user'      => 'manage','sb::Admin::User',     2,'_menu11'  ],
		['build'     => 'manage','sb::Admin::Build',    0,'_menu12'  ],
		['bm'        => '',      'sb::Admin::Quickpost',0,'_'        ],
	);
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'mode'  => undef,
		'regi'  => undef,
		'user'  => undef,
		'users' => {},
		'list'  => [],
		'lock'  => undef,
		@_
	);
	$self->generate_accessor('mode','regi','user','users','list');
	return $self;
}
sub run
{
	my $class = shift;
	my %param = (
		@_,
		'users' => {sb::Data->load_as_hash('User')},
		'list'  => [
			$class->default_menulist(),
			sb->get->plugin->load_admin_module()
		],
	);
	# $self works as admin module loader
	my $self = $class->new(%param);
	my $error = $self->_check_mode();
	# mapping hash for admin module from the list
	my %lists = map { $_->[MENU_MODE] => $_->[MENU_MODULE] } @{$self->list};
	my $module = $lists{$self->mode};
	# handling errors
	return $self->error(sb->string('Failed to load database, please check database settings.')) if (!sb->get->driver);
	return $self->error(sb->string('Unknown error.')) if (!$module or !_load_module($module));
	# load and execute the module
	my $admin = $module->new(%$self);
	eval {
		$self->output($admin->callback('message'=>$error));
	};
	$self->error($@) if ($@);
	$admin->bye() if ($admin);
}
# --------------------------------------
# // override base class - template parts
sub common_parts
{
	my $self = shift;
	my $temp = shift;
	return if (!$temp or !$self);
	$self->SUPER::common_parts($temp);
	$temp->tag('sb_site_cgi'=>$self->script_path);
	{ # required javascripts
		my $js_tag = '';
		foreach my $js ( $self->js_paths )
		{
			next if (!$js);
			$js_tag .= "\n" . sprintf('<script type="text/javascript" src="%s"></script>',$js)
		}
		$temp->tag('sb_site_js'=>$js_tag);
	}
	$temp->tag('sb_body_id'=>$self->conf->value('basic_admntag'));
	$temp->tag('sb_extra_style'=>$self->extra_styles());
	return;
}
sub resource_dir
{
	my $self = shift;
	return $self->conf->value('dir_temp') . $self->lang->language . '/';
}
sub static_path
{
	my $self = shift;
	return $self->conf->value('srv_temp');
}
sub style_path
{
	my $self = shift;
	return $self->static_path . CSS_NAME;
}
# --------------------------------------
# // public functions - actions
sub create_category
{
	my $self = shift;
	my %param = (
		'name' => undef, # [required][CHAR] category name
		'main' => undef, # [optional][NUM.] parent category (id)
		'sub'  => undef, # [optional][SEL.] create as child? 1 : yes
		'text' => undef, # [optional][CHAR] description
		'num'  => 0,     # [optional][NUM.] number of article
		@_
	);
	return if (! defined($param{'name'}));
	my $parent = undef;
	if ($param{'sub'} and $param{'main'} ne '')
	{
		$parent = sb::Data->load('Category','id'=>$param{'main'});
	}
	# --- now check the existance of the category
	my @same_cats = sb::Data->load('Category','cond'=>{'name'=>$param{'name'}});
	foreach my $cat (@same_cats)
	{
		return $cat if ($parent and $cat->main eq $param{'min'});
		return $cat if (!$parent and $cat->main eq '');
	}
	my $new_category = sb::Data->add('Category',
		'main' => ($parent) ? $parent->id : '',
		'name' => $param{'name'},
		'url'  => '',
		'text' => $param{'text'},
		'temp' => ($parent) ? $parent->temp : -1,   # inherit settings from parent
		'dir'  => ($parent) ? $parent->dir : undef, # inherit settings from parent
		'disp' => ($parent) ? $parent->disp : '',   # inherit settings from parent
		'sub'  => '',
		'num'  => $param{'num'},
	);
	sb::Data->update($new_category);
	if ($parent)
	{ # we should update parent category as well
		$parent->add_sub($new_category->id);
		sb::Data->update($parent);
	}
	return $new_category;
}
sub upload_file
{
	my $self = shift;
	my %param = (
		'max'  => $self->conf->value('basic_max_img'),  # [optional][NUM.] maximum number to upload
		'base' => $self->conf->value('conf_dir_base'),  # [optional][TEXT] base directory
		'name' => $self->conf->value('conf_imagename'), # [optional][SEL.] flag for using fixed name
		'over' => undef,                                # [optional][SEL.] flag for overwriting image
		'data' => 1,                                    # [optional][SEL.] flag for updating DB
		@_
	);
	my $dir = $param{'base'} . $self->cgi->value('upload_dir');
	my $filenum = ($param{'data'}) ? $self->_get_next_imageid() : undef;
	my @upload = ();
	for (my $i=0;$i<$param{'max'};$i++)
	{
		my $file = $self->cgi->value('upload_file' . $i);
		my $name = $self->cgi->value('upload_name' . $i);
		next if ($file eq '');
		my $loader = sb::File::Uploader->new();
		$loader->path($dir); # set dir to upload
		$loader->set_param( # prepare to upload
			'label'  => 'upload_file' . $i,
			'header' => [$self->cgi->content_list]
		);
		next if ($loader->uploaded_name eq ''); # failed to parse
		next if ($loader->uploaded_extension eq ''); # it's a not supported file
		$name = $loader->uploaded_name if ($name eq '');
		if ( defined($filenum) and !$param{'name'} )
		{ # put the prefix to the filename
			my $filename  = $loader->uploaded_name;
			if ($filename =~ /[^a-zA-Z0-9_\-\.]/)
			{
				$filename = sb::Text->new($filename)->format('md5','encode');
				$filename = substr($filename,0,64) if (length($filename) > 64);
				$filename .= $loader->uploaded_extension;
			}
			$loader->name(PREFIX_NAME . $filenum++ . '_' . $filename); # set name to upload
		}
		else
		{ # keep the name of file to upload
			$loader->name($loader->uploaded_name); # set name to upload
		}
		next if ( $loader->exists and !$param{'over'} ); # not overwrite mode, we should check existence
		next if ( !$loader->upload($file) ); # uploading file - it's failed if false is returned.
		push(@upload,{
			'name' => $name,
			'file' => sb::File::Object->new('fullpath'=>$loader->fullpath),
			'over' => $loader->exists,
		});
	} # end of for (my $i=0;$i<$param{'max'};$i++)
	if ( $param{'data'} )
	{
		my @files = ();
		my $id = ($self->user) ? $self->user->id : 0;
		for my $obj ( @upload )
		{
			my $file = undef;
			if ( $obj->{'over'} )
			{
				$file = sb::Data->load('Image',
					'cond' => {
						'dir'  => $self->cgi->value('upload_dir'),
						'file' => $obj->{'file'}->name()
					},
					'num' => 1,
				);
			}
			if ( !$file )
			{
				$file = sb::Data->add('Image');
			}
			next if ( !$file );
			$file->name( $obj->{'name'} );
			$file->auth( $id );
			$file->date( $self->time->data );
			$file->tz( $self->conf->value('conf_timezone') );
			$file->set_file($obj->{'file'},$self->cgi->value('upload_thumb'),$param{'name'});
			push(@files,$file);
		} # end of for my $obj ( @upload )
		sb::Data->update(@files) if ( @files );
		return @files;
	} # end of if ( $param{'data'} )
	return @upload;
}
sub set_main
{
	my $self = shift;
	my $content = shift;
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $mode = $self->mode;
	$mode = 'bm' if ($self->mode ne 'bm' and $self->cgi->value('bm') eq 'on');
	$self->common_parts($temp);
	$self->_set_menu($temp);
	$self->_set_title($temp);
	$temp->num(0);
	$temp->tag('sb_mainbody'=>$content);
	$temp->block('sb_mainbody'=>1) if ($mode ne 'bm');
	$temp->block('sb_bookmarklet'=>1) if ($mode eq 'bm');
	$temp->block('sb_login'=>0);
	return $temp->output;
}
sub get_editor
{
	my $self = shift;
	my $class = $self->editor_name();
	return $class if ($class ne DEFAULT_EDITOR and sb::load_module($class));
	sb::load_module(DEFAULT_EDITOR);
	return DEFAULT_EDITOR;
}
sub has_permission
{
	my $self = shift;
	my %param = (
		'level' => undef,
		'id'  => undef,
		@_
	);
	return if ( !$self->users );
	return if ( !$self->user );
	if ( defined($param{'id'}) )
	{
		return 1 if ( $self->user->id == $param{'id'} );
		return 1 if ( !$self->users->{$param{'id'}} );
		return 1 if ( $self->user->stat < $self->users->{$param{'id'}}->stat );
	}
	elsif ( defined($param{'level'}) )
	{
		return 1 if ( $self->user->stat <= MAX_LEVEL - $param{'level'} );
	}
	return;
}
# --------------------------------------
# // public functions - template parts
sub script_path
{
	my $self = shift;
	return $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_admn');
}
sub js_paths
{ # it should return javascript paths as array
	my $self = shift;
	return $self->static_path . JSFILE_NAME;
}
sub extra_styles
{
	my $self = shift;
	my $output = '';
	foreach my $name ( $self->extra_styleset )
	{
		my $href = $self->static_path . 'ext_' . $name . '.css';
		my $rel = ($self->user and $self->user->ad_css eq $name) ? 'stylesheet' : 'alternate stylesheet';
		$output .= "\n" . sprintf(EXTRA_STYLE,$rel,$href,$name);
	}
	return $output;
}
sub extra_styleset
{
	my $self = shift;
	opendir(RSRC_DIR,$self->conf->value('dir_rsrc'));
	my @css = grep { s/^ext_(\w+?)\.css$/$1/ } readdir(RSRC_DIR);
	closedir(RSRC_DIR);
	return @css;
}
sub set_message
{
	my $self = shift;
	my $temp = shift;
	my $msg = shift;
	return if ($msg eq '');
	$msg =~ s|\n|<br />\n|g;
	$temp->num(0);
	$temp->tag('sb_process_message'=>$msg);
	$temp->block('sb_process_message'=>1);
}
# --------------------------------------
# // public functions - declaration
sub bye
{
	my $self = shift;
	$self->{'lock'}->unlock() if ($self->{'lock'});
}
sub callback
{
	my $self = shift;
	die(sb->string('Unknown error.') . "\n");
}
sub set_onload
{
	return;
}
sub editor_name
{
	my $self = shift;
	return DEFAULT_EDITOR;
}
sub title_name
{
	my $self = shift;
	return sb->string(' &gt; Quickpost') if ($self->mode eq 'bm' or $self->cgi->value('bm') eq 'on');
	return;
}
# --------------------------------------
# // private functions - deprecated
sub get_script_path
{
	script_path(@_);
}
sub get_parts_dir
{
	resource_dir(@_);
}
sub common_template_parts
{
	common_parts(@_);
}
sub upload_image
{
	upload_file(@_);
}
# --------------------------------------
# // private functions - template parts
sub _set_title
{
	my $self = shift;
	my $temp = shift;
	return if (!$temp or !$self);
	my $weblog = sb::Data->load('Weblog','id'=>0);
	my $title = '[' . $sb::ACRONYM . ']';
	$title .= ' ' . $weblog->title if ($weblog);
	$title .= $self->title_name();
	my $mode = ($self->mode ne '') ? $self->mode : 'login';
	$mode = 'bm' if ($self->cgi->value('bm') eq 'on');
	$temp->num(0);
	$temp->tag('sb_site_title'=>$title);
	$temp->tag('sb_body_class'=>($mode ne 'bm') ? 'main' : 'bm');
	$temp->tag('sb_body_class'=>$self->set_onload) if ($self->mode eq 'bm');
	return;
}
sub _set_menu
{
	my $self = shift;
	my $temp = shift;
	my $path = $self->script_path;
	return if ($self->mode eq 'bm' or $self->cgi->value('bm') eq 'on');
	my %blocks = ();
	my $help_option = undef;
	foreach my $menu ( @{$self->list} )
	{
		next if (!$self->has_permission('level'=>$menu->[MENU_LEVEL]));
		my $mode = $menu->[MENU_MODE];
		$help_option = $menu->[MENU_HELP] if ($mode eq $self->mode);
		next if ($menu->[MENU_TYPE] eq '');
		my $type = 'sb_' . $menu->[MENU_TYPE] . 'menu';
		$temp->num( $blocks{$type}++ );
		$temp->tag($type . '_class' => ($mode eq $self->mode) ? ' class="current"' : '');
		$temp->tag($type . '_url'   => $path . '?__mode=' . $mode);
		$temp->tag($type . '_id'    => $mode);
		$temp->tag($type . '_name'  => sb->string('mode_' . $mode));
	}
	foreach my $name ( keys %blocks )
	{
		$temp->block($name => $blocks{$name});
		$temp->block($name . '_body' => 1) if ($blocks{$name} > 0);
	}
	# --- rest of menus
	$temp->num(0);
	$temp->tag('sb_head_help_url'=>$path . '?__mode=help&amp;help=' . $help_option);
	if ($self->user)
	{
		$temp->tag('sb_head_user_url'=>$path . '?__mode=profile');
		$temp->tag('sb_head_user_name'=>$self->user->real);
		$temp->block('sb_headuser'=>1);
	}
	$temp->block('sb_mainhead'=>1);
	return;
}
# --------------------------------------
# // private functions
sub _get_next_imageid
{
	my $self = shift;
	my $driver = sb->get->driver;
	return $driver->get_nextid('sb::Data::Image');
}
sub _load_module
{
	my $class = shift;
	return 1 if ( $class->isa('sb::App::Admin') );
	return sb::load_module($class);
}
sub _check_mode
{
	my $self = shift;
	my $pass = $self->cgi->value('__pass');
	my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_admntag') . COOKIE_USER);
	my %level_of = map { $_->[MENU_MODE] => $_->[MENU_LEVEL] } @{$self->list};
	$self->mode($self->cgi->value('__mode'));
	$self->regi($self->cgi->value('__regi'));
	if ($self->regi ne '' and $self->conf->value('basic_ref_check'))
	{ # we should check HTTP_REFERER
		$self->mode('login') if (index($self->cgi->referer,$self->script_path) == -1);
	}
	$self->mode('login') if ( !$self->mode or !defined($level_of{$self->mode}) ); # login is default mode
	return if ($self->mode eq 'login' and $pass eq ''); # in login mode, so we can skip
	# --- get the logined user name and update cookie information
	my $name = ( $self->cgi->value('__user') ) ? $self->cgi->value('__user') : $cookie->{'user'};
	$self->cgi->set_cookie(
		'time'   => $self->time->data,
		'name'   => $self->conf->value('basic_admntag') . COOKIE_USER,
		'expire' => $self->conf->value('basic_admn_expire'),
		'path'   => $self->conf->value('conf_srv_cgi'),
		'data'   => {'user' => $name},
	);
	# --- check through all users until match the logined user
	foreach my $id ( keys %{$self->users} )
	{
		my $user = $self->users()->{$id};
		next if ( $user->name ne $name );
		my $session = sb::Session->new(
			'time'   => $self->time->data,
			'cgi'    => $self->cgi,
			'key'    => $user->id,
			'path'   => $self->script_path,
			'name'   => $self->conf->value('basic_sessiontag'),
			'expire' => $self->conf->value('basic_admn_expire'),
		);
		$self->user($user);
		if (!$self->has_permission('level' => $level_of{$self->mode}))
		{
			$session->finish;
			$self->mode('login');
			return sb->string('Not authorized.');
		}
		if ( $pass ne '' and $user->check_pass($pass) )
		{ # pass the password check
			$session->start;
			$self->mode('home') if ($self->mode eq 'login' or $self->mode eq 'logout');
			$self->cgi->set_cookie(
				'time'   => $self->time->data,
				'name'   => $self->conf->value('basic_logtag'),
				'expire' => COOKIE_FOR_LOG,
				'path'   => $self->conf->value('conf_srv_cgi'),
				'data'   => {'check' => $self->conf->value('basic_cookiekey')},
			);
			return;
		}
		elsif ($pass ne '')
		{
			$session->finish;
			$self->mode('login');
			return sb->string('Incorrect password.');
		}
		if ( $self->mode eq 'logout' )
		{ # logout procedure
			$session->finish;
			$self->mode('login');
			return sb->string('You have been logged out successfully.');
		}
		if (!$session->check)
		{ # session is expired
			$self->mode('login');
			return sb->string('Timeout error.  Please log in again to continue.');
		}
		$session->start;
		return;
	}
	$self->mode('login');
	return sb->string('No such user.');
}
1;
__END__
